@@ -7,7 +7,7 @@
    This file is part of the GNUstep Base Library.
    
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
@@ -16,23 +16,40 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */ 
 
 #ifndef __NSData_h_GNUSTEP_BASE_INCLUDE
 #define __NSData_h_GNUSTEP_BASE_INCLUDE
+#import	<GNUstepBase/GSVersionMacros.h>
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSRange.h>
-#include <Foundation/NSSerialization.h>
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSRange.h>
+#import	<Foundation/NSSerialization.h>
 
-#ifndef STRICT_OPENSTEP
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
+@class	NSError;
 @class	NSURL;
 #endif
 
+#if OS_API_VERSION(100400,GS_API_LATEST) 
+enum {
+  NSMappedRead = 1,
+  NSUncachedRead = 2
+};
+
+enum {
+  NSAtomicWrite = 1
+};
+#endif
+
 @interface NSData : NSObject <NSCoding, NSCopying, NSMutableCopying>
 
 // Allocating and Initializing a Data Object
@@ -42,14 +59,14 @@
 	      length: (unsigned int)length;
 + (id) dataWithBytesNoCopy: (void*)bytes
 		    length: (unsigned int)length;
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 + (id) dataWithBytesNoCopy: (void*)aBuffer
 		    length: (unsigned int)bufferSize
 	      freeWhenDone: (BOOL)shouldFree;
 #endif
 + (id) dataWithContentsOfFile: (NSString*)path;
 + (id) dataWithContentsOfMappedFile: (NSString*)path;
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 + (id) dataWithContentsOfURL: (NSURL*)url;
 #endif
 + (id) dataWithData: (NSData*)data;
@@ -57,14 +74,14 @@
 	      length: (unsigned int)bufferSize;
 - (id) initWithBytesNoCopy: (void*)aBuffer
 		    length: (unsigned int)bufferSize;
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (id) initWithBytesNoCopy: (void*)aBuffer
 		    length: (unsigned int)bufferSize
 	      freeWhenDone: (BOOL)shouldFree;
 #endif
 - (id) initWithContentsOfFile: (NSString*)path;
 - (id) initWithContentsOfMappedFile: (NSString*)path;
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (id) initWithContentsOfURL: (NSURL*)url;
 #endif
 - (id) initWithData: (NSData*)data;
@@ -90,7 +107,7 @@
 - (BOOL) writeToFile: (NSString*)path
 	  atomically: (BOOL)useAuxiliaryFile;
 
-#ifndef	STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (BOOL) writeToURL: (NSURL*)anURL atomically: (BOOL)flag;
 #endif
 
@@ -113,9 +130,19 @@
 		   count: (unsigned int)numInts
 		 atIndex: (unsigned int)index;
 
+#if OS_API_VERSION(100400,GS_API_LATEST) 
+/** Not implemented */
+- (BOOL) writeToFile: (NSString *)path
+             options: (NSUInteger)writeOptionsMask
+               error: (NSError **)errorPtr;
+/** Not implemented */
+- (BOOL) writeToURL: (NSURL *)url
+            options: (NSUInteger)writeOptionsMask
+              error: (NSError **)errorPtr;
+#endif
 @end
 
-#ifndef NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
 /**
  * Extension methods for the NSData class- mainly conversion utilities.
@@ -294,7 +321,7 @@
 
 - (void) replaceBytesInRange: (NSRange)aRange
 		   withBytes: (const void*)bytes;
-#ifndef STRICT_OPENSTEP
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (void) replaceBytesInRange: (NSRange)aRange
 		   withBytes: (const void*)bytes
 		      length: (unsigned int)length;
@@ -319,7 +346,7 @@
 
 @end
 
-#ifndef	NO_GNUSTEP
+#if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 
 @interface NSMutableData (GNUstepExtensions)
 /*
@@ -344,10 +371,8 @@
 @end
 #endif
 
-/*
-  Local Variables:
-  mode: ObjC
-  End:
-  */
+#if	defined(__cplusplus)
+}
+#endif
 
 #endif /* __NSData_h_GNUSTEP_BASE_INCLUDE */
