@@ -7,7 +7,7 @@
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
@@ -16,20 +16,25 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */ 
 
 #ifndef __NSKeyValueObserving_h_GNUSTEP_BASE_INCLUDE
 #define __NSKeyValueObserving_h_GNUSTEP_BASE_INCLUDE
-
-#include	<Foundation/NSObject.h>
+#import	<GNUstepBase/GSVersionMacros.h>
 
 #if OS_API_VERSION(100300,GS_API_LATEST) && GS_API_VERSION(010200,GS_API_LATEST)
 
-@class NSArray;
+#import	<Foundation/NSObject.h>
+#import	<Foundation/NSArray.h>
+
+#if	defined(__cplusplus)
+extern "C" {
+#endif
+
 @class NSIndexSet;
 @class NSSet;
 @class NSString;
@@ -37,6 +42,10 @@
 typedef enum {
   NSKeyValueObservingOptionNew = 1,
   NSKeyValueObservingOptionOld = 2
+#if OS_API_VERSION(100500,GS_API_LATEST)
+,  NSKeyValueObservingOptionInitial = 4,
+  NSKeyValueObservingOptionPrior = 8
+#endif
 } NSKeyValueObservingOptions;
 
 typedef enum {
@@ -57,6 +66,9 @@ GS_EXPORT NSString *const NSKeyValueChangeIndexesKey;
 GS_EXPORT NSString *const NSKeyValueChangeKindKey;
 GS_EXPORT NSString *const NSKeyValueChangeNewKey;
 GS_EXPORT NSString *const NSKeyValueChangeOldKey;
+#if OS_API_VERSION(100500,GS_API_LATEST)
+GS_EXPORT NSString *const NSKeyValueChangeNotificationIsPriorKey;
+#endif
 
 /* Given that the receiver has been registered as an observer
  * of the value at a key path relative to an object,
@@ -198,10 +210,10 @@ GS_EXPORT NSString *const NSKeyValueChangeOldKey;
 
 /**
  * Tells the observing system that when NSKeyValueObserverNotification
- * protocol messages are sent for any key in the keys array, they should
- * also be sent for dependentKey.
+ * protocol messages are sent for any key in the triggerKeys array,
+ * they should also be sent for dependentKey.
  */
-+ (void) setKeys: (NSArray*)keys
++ (void) setKeys: (NSArray*)triggerKeys
 triggerChangeNotificationsForDependentKey: (NSString*)dependentKey;
 
 /**
@@ -220,6 +232,11 @@ triggerChangeNotificationsForDependentKey: (NSString*)dependentKey;
 
 @end
 
+#if	defined(__cplusplus)
+}
 #endif
-#endif /* __NSKeyValueObserving_h_GNUSTEP_BASE_INCLUDE */
+
+#endif	/* 100300 */
+
+#endif	/* __NSKeyValueObserving_h_GNUSTEP_BASE_INCLUDE */
 
