@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDictionary class reference</title>
-   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
+   $Date: 2008-07-12 14:30:27 +0100 (Sat, 12 Jul 2008) $ $Revision: 26761 $
    */
 
 #include "config.h"
@@ -437,8 +437,32 @@ static SEL	appSel;
     {
       GS_BEGINIDBUF(o, objectCount*2);
 
-      [objects getObjects: o];
-      [keys getObjects: o + objectCount];
+      if ([objects isProxy])
+	{
+	  unsigned	i;
+
+	  for (i = 0; i < objectCount; i++)
+	    {
+	      o[i] = [objects objectAtIndex: i];
+	    }
+	}
+      else
+	{
+          [objects getObjects: o];
+	}
+      if ([keys isProxy])
+	{
+	  unsigned	i;
+
+	  for (i = 0; i < objectCount; i++)
+	    {
+	      o[objectCount + i] = [keys objectAtIndex: i];
+	    }
+	}
+      else
+	{
+          [keys getObjects: o + objectCount];
+	}
       self = [self initWithObjects: o
 			   forKeys: o + objectCount
 			     count: objectCount];
@@ -899,7 +923,17 @@ compareIt(id o1, id o2, void* context)
       id	result;
       GS_BEGINIDBUF(obuf, c);
 
-      [keys getObjects: obuf];
+      if ([keys isProxy])
+	{
+	  for (i = 0; i < c; i++)
+	    {
+	      obuf[i] = [keys objectAtIndex: i];
+	    }
+	}
+      else
+	{
+          [keys getObjects: obuf];
+	}
       for (i = 0; i < c; i++)
 	{
 	  id o = (*myObj)(self, objSel, obuf[i]);
@@ -1245,7 +1279,19 @@ compareIt(id o1, id o2, void* context)
       IMP	remObj = [self methodForSelector: remSel];
       GS_BEGINIDBUF(keys, c);
 
-      [keyArray getObjects: keys];
+      if ([keyArray isProxy])
+	{
+	  unsigned	i;
+
+	  for (i = 0; i < c; i++)
+	    {
+	      keys[i] = [keyArray objectAtIndex: i];
+	    }
+	}
+      else
+	{
+	  [keyArray getObjects: keys];
+	}
       while (c--)
 	{
 	  (*remObj)(self, remSel, keys[c]);
