@@ -7,7 +7,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +20,7 @@
    Boston, MA 02111 USA.
 
    <title>NSMessagePortNameServer class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 #include "Foundation/NSPortNameServer.h"
@@ -29,6 +29,7 @@
 #include "Foundation/NSDebug.h"
 #include "Foundation/NSException.h"
 #include "Foundation/NSLock.h"
+#include "Foundation/NSDistributedLock.h"
 #include "Foundation/NSMapTable.h"
 #include "Foundation/NSPathUtilities.h"
 #include "Foundation/NSPort.h"
@@ -77,16 +78,17 @@ static NSMapTable portToNamesMap;
 
 
 @interface NSMessagePortNameServer (private)
-+(NSString *) _pathForName: (NSString *)name;
++ (NSDistributedLock*) _fileLock;
++ (NSString *) _pathForName: (NSString *)name;
 @end
 
 
 static void clean_up_names(void)
 {
-  NSMapEnumerator mEnum;
-  NSMessagePort	*port;
-  NSString	*name;
-  BOOL	unknownThread = GSRegisterCurrentThread();
+  NSMapEnumerator	mEnum;
+  NSMessagePort		*port;
+  NSString		*name;
+  BOOL			unknownThread = GSRegisterCurrentThread();
   CREATE_AUTORELEASE_POOL(arp);
 
   mEnum = NSEnumerateMapTable(portToNamesMap);
@@ -139,21 +141,73 @@ static void clean_up_names(void)
   return defaultServer;
 }
 
++ (NSDistributedLock*) _fileLock
+{
+  NSDistributedLock	*dl;
 
+  dl = [NSDistributedLock lockWithPath: [self _pathForName: nil]];
+  if ([dl tryLock] == NO)
+    {
+      NSDate	*limit = [NSDate dateWithTimeIntervalSinceNow: 2.0];
+
+      while (dl != nil && [dl tryLock] == NO)
+	{
+	  if ([limit timeIntervalSinceNow] > 0.0)
+	    {
+	      [NSThread sleepForTimeInterval: 0.1];
+	    }
+	  else
+	    {
+              if ([[dl lockDate] timeIntervalSinceNow] < -15.0)
+                {
+                  NS_DURING
+                    {
+                      [dl breakLock];
+                    }
+                  NS_HANDLER
+                    {
+                      NSLog(@"Failed to break lock on names for "
+                        @"NSMessagePortNameServer: %@", localException);
+                      dl = nil;
+                    }
+                  NS_ENDHANDLER
+                }
+              else
+                {
+                  NSLog(@"Failed to lock names for NSMessagePortNameServer");
+                  dl = nil;
+                }
+	    }
+	}
+    }
+  return dl;
+}
+
+/* Return the full path for the supplied port name or, if it's nil,
+ * the path for the distributed lock protecting all names.
+ */
 + (NSString *) _pathForName: (NSString *)name
 {
   static NSString	*base_path = nil;
   NSString		*path;
   NSData		*data;
 
-  /*
-   * Make sure name is representable as a filename ... assume base64 encoded
-   * strings are valid on all filesystems.
-   */
-  data = [name dataUsingEncoding: NSUTF8StringEncoding];
-  data = [GSMimeDocument encodeBase64: data];
-  name = [[NSString alloc] initWithData: data encoding: NSASCIIStringEncoding];
-  AUTORELEASE(name);
+  if (name == nil)
+    {
+      name = @"lock";
+    }
+  else
+    {
+      /*
+       * Make sure name is representable as a filename ... assume base64 encoded
+       * strings are valid on all filesystems.
+       */
+      data = [name dataUsingEncoding: NSUTF8StringEncoding];
+      data = [GSMimeDocument encodeBase64: data];
+      name = [[NSString alloc] initWithData: data
+				   encoding: NSASCIIStringEncoding];
+      AUTORELEASE(name);
+    }
   [serverLock lock];
   if (!base_path)
     {
@@ -260,9 +314,10 @@ static void clean_up_names(void)
 - (NSPort*) portForName: (NSString *)name
 		 onHost: (NSString *)host
 {
-  NSString	*path;
-  FILE		*f;
-  char		socket_path[512];
+  NSDistributedLock	*dl;
+  NSString		*path;
+  FILE			*f;
+  char			socket_path[512];
 
   NSDebugLLog(@"NSMessagePort", @"portForName: %@ host: %@", name, host);
 
@@ -278,8 +333,14 @@ static void clean_up_names(void)
     }
 
   path = [[self class] _pathForName: name];
+  if ((dl = [[self class] _fileLock]) == nil)
+    {
+      [NSException raise: NSGenericException
+		  format: @"Failed to lock names for NSMessagePortNameServer"];
+    }
   if (![[self class] _livePort: path])
     {
+      [dl unlock];
       NSDebugLLog(@"NSMessagePort", @"not a live port");
       return nil;
     }
@@ -287,6 +348,7 @@ static void clean_up_names(void)
   f = fopen([path fileSystemRepresentation], "rt");
   if (!f)
     {
+      [dl unlock];
       NSDebugLLog(@"NSMessagePort", @"can't open file (%m)");
       return nil;
     }
@@ -296,6 +358,7 @@ static void clean_up_names(void)
   fclose(f);
 
   NSDebugLLog(@"NSMessagePort", @"got %s", socket_path);
+  [dl unlock];
 
   return [NSMessagePort _portWithName: (unsigned char*)socket_path
 			     listener: NO];
@@ -306,6 +369,7 @@ static void clean_up_names(void)
 {
   int			fd;
   unsigned char		buf[32];
+  NSDistributedLock	*dl;
   NSString		*path;
   const unsigned char	*socket_name;
   NSMutableArray	*a;
@@ -316,13 +380,17 @@ static void clean_up_names(void)
       [NSException raise: NSInvalidArgumentException
 		  format: @"Attempted to register a non-NSMessagePort (%@)",
 	port];
-      return NO;
     }
 
   path = [[self class] _pathForName: name];
-
+  if ((dl = [[self class] _fileLock]) == nil)
+    {
+      [NSException raise: NSGenericException
+		  format: @"Failed to lock names for NSMessagePortNameServer"];
+    }
   if ([[self class] _livePort: path])
     {
+      [dl unlock];
       NSDebugLLog(@"NSMessagePort", @"fail, is a live port (%@)", name);
       return NO;
     }
@@ -330,6 +398,7 @@ static void clean_up_names(void)
   fd = open([path fileSystemRepresentation], O_CREAT|O_EXCL|O_WRONLY, 0600);
   if (fd < 0)
     {
+      [dl unlock];
       NSDebugLLog(@"NSMessagePort", @"fail, can't open file (%@) for %@",
 	path, name);
       return NO;
@@ -355,17 +424,25 @@ static void clean_up_names(void)
 
   [a addObject: [name copy]];
   [serverLock unlock];
+  [dl unlock];
 
   return YES;
 }
 
 - (BOOL) removePortForName: (NSString *)name
 {
-  NSString	*path;
+  NSDistributedLock	*dl;
+  NSString		*path;
 
   NSDebugLLog(@"NSMessagePort", @"removePortForName: %@", name);
   path = [[self class] _pathForName: name];
+  if ((dl = [[self class] _fileLock]) == nil)
+    {
+      [NSException raise: NSGenericException
+		  format: @"Failed to lock names for NSMessagePortNameServer"];
+    }
   unlink([path fileSystemRepresentation]);
+  [dl unlock];
   return YES;
 }
 
@@ -405,29 +482,36 @@ static void clean_up_names(void)
 {
   FILE			*f;
   char			socket_path[512];
+  NSDistributedLock	*dl;
   NSString		*path;
   const unsigned char	*port_path;
 
   NSDebugLLog(@"NSMessagePort", @"removePort: %@  forName: %@", port, name);
 
   path = [[self class] _pathForName: name];
-
+  if ((dl = [[self class] _fileLock]) == nil)
+    {
+      [NSException raise: NSGenericException
+		  format: @"Failed to lock names for NSMessagePortNameServer"];
+    }
   f = fopen([path fileSystemRepresentation], "rt");
   if (!f)
-    return YES;
-
+    {
+      [dl unlock];
+      return YES;
+    }
   fgets(socket_path, sizeof(socket_path), f);
-  if (strlen(socket_path) > 0) socket_path[strlen(socket_path) - 1] = 0;
-
+  if (strlen(socket_path) > 0)
+    {
+      socket_path[strlen(socket_path) - 1] = 0;
+    }
   fclose(f);
-
   port_path = [(NSMessagePort *)port _name];
-
   if (!strcmp((char*)socket_path, (char*)port_path))
     {
       unlink([path fileSystemRepresentation]);
     }
-
+  [dl unlock];
   return YES;
 }
 
