@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSPage class reference</title>
-   $Date: 2007-09-14 05:39:44 -0600 (Fri, 14 Sep 2007) $ $Revision: 25483 $
+   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
    */
 
 #include "config.h"
@@ -38,11 +38,6 @@
 #include <mach.h>
 #endif
 
-#if __linux__
-#include <linux/kernel.h>
-#include <linux/sys.h>
-#endif
-
 #ifdef __CYGWIN__
 #include <malloc.h>
 #endif
