@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSRunLoop class reference</title>
-   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
+   $Date: 2008-07-12 14:30:27 +0100 (Sat, 12 Jul 2008) $ $Revision: 26761 $
 */
 
 #include "config.h"
@@ -360,7 +360,17 @@ static inline BOOL timerInvalidated(NSTimer *t)
 						  delay: seconds];
       [[loop _timedPerformers] addObject: item];
       RELEASE(item);
-      [modes getObjects: marray];
+      if ([modes isProxy])
+	{
+	  for (i = 0; i < count; i++)
+	    {
+	      marray[i] = [modes objectAtIndex: i];
+	    }
+	}
+      else
+	{
+          [modes getObjects: marray];
+	}
       for (i = 0; i < count; i++)
 	{
 	  [loop addTimer: item->timer forMode: marray[i]];
@@ -1405,7 +1415,19 @@ static inline BOOL timerInvalidated(NSTimer *t)
 						 argument: argument
 						    order: order];
 
-      [modes getObjects: array];
+      if ([modes isProxy])
+	{
+	  unsigned	i;
+
+	  for (i = 0; i < count; i++)
+	    {
+	      array[i] = [modes objectAtIndex: i];
+	    }
+	}
+      else
+	{
+          [modes getObjects: array];
+	}
       while (count-- > 0)
 	{
 	  NSString	*mode = array[count];
