@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSSerializer class reference</title>
-   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
+   $Date: 2008-07-12 14:30:27 +0100 (Sat, 12 Jul 2008) $ $Revision: 26761 $
    */
 
 #include "config.h"
@@ -280,7 +280,17 @@ serializeToInfo(id object, _NSSerializerInfo* info)
 	  id		objects[count];
 	  unsigned int	i;
 
-	  [object getObjects: objects];
+	  if ([object isProxy])
+	    {
+	      for (i = 0; i < count; i++)
+		{
+		  objects[i] = [object objectAtIndex: i];
+		}
+	    }
+	  else
+	    {
+	      [object getObjects: objects];
+	    }
 	  for (i = 0; i < count; i++)
 	    {
 	      serializeToInfo(objects[i], info);
