@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSSet class reference</title>
-   $Date: 2008-06-12 11:44:00 +0100 (Thu, 12 Jun 2008) $ $Revision: 26630 $
+   $Date: 2008-07-12 14:30:27 +0100 (Sat, 12 Jul 2008) $ $Revision: 26761 $
    */
 
 #include "config.h"
@@ -381,7 +381,19 @@ static Class NSMutableSet_concrete_class;
     {
       GS_BEGINIDBUF(objs, count);
 
-      [other getObjects: objs];
+      if ([other isProxy])
+	{
+	  unsigned	i;
+
+	  for (i = 0; i < count; i++)
+	    {
+	      objs[i] = [other objectAtIndex: i];
+	    }
+	}
+      else
+	{
+          [other getObjects: objs];
+	}
       self = [self initWithObjects: objs count: count];
       GS_ENDIDBUF();
       return self;
