@@ -246,6 +246,75 @@ cifframe_from_info (NSArgumentInfo *info, int numargs, void **retval)
   return cframe;
 }
 
+/* NB. this must match the code in cifframe_from_info() so that it
+ * returns the offset for the returne value in the cframe.
+ */
+unsigned
+retval_offset_from_info (NSArgumentInfo *info, int numargs)
+{
+  unsigned      size = sizeof(cifframe_t);
+  unsigned      align = __alignof(double);
+  unsigned      type_offset = 0;
+  unsigned      offset = 0;
+  int           i;
+  ffi_type      *arg_types[numargs];
+  ffi_type      *rtype;
+
+  /* FIXME: in cifframe_type, return values/arguments that are structures
+     have custom ffi_types with are allocated separately. We should allocate
+     them in our cifframe so we don't leak memory. Or maybe we could
+     cache structure types? */
+  rtype = cifframe_type(info[0].type, NULL);
+  if (rtype == 0 || (rtype->size == 0 && rtype->elements == NULL))
+    {
+      return 0;
+    }
+
+  for (i = 0; i < numargs; i++)
+    {
+      arg_types[i] = cifframe_type(info[i+1].type, NULL);
+    }
+
+  if (numargs > 0)
+    {
+      if (size % align != 0)
+        {
+          size += align - (size % align);
+        }
+      type_offset = size;
+      /* Make room to copy the arg_types */
+      size += sizeof(ffi_type *) * numargs;
+      if (size % align != 0)
+        {
+          size += align - (size % align);
+        }
+      offset = size;
+      size += numargs * sizeof(void*);
+      if (size % align != 0)
+        {
+          size += (align - (size % align));
+        }
+      for (i = 0; i < numargs; i++)
+        {
+          if (arg_types[i]->elements)
+            size += cifframe_guess_struct_size(arg_types[i]);
+          else
+            size += arg_types[i]->size;
+
+          if (size % align != 0)
+            {
+              size += (align - size % align);
+            }
+        }
+    }
+
+  if (size % align != 0)
+    {
+      size += (align - size % align);
+    }
+  return size;
+}
+
 void
 cifframe_set_arg(cifframe_t *cframe, int index, void *buffer, int size)
 {
